#include <algorithm>
#include <iostream>
#include <numeric>
#include <cassert>
#include <vector>
#include <bitset>

#pragma GCC optimize ("Ofast,fast-math,unroll_loops")
#pragma GCC target ("sse4")

typedef long long llong;
const int MAXS = 250000 + 10;
const int MAXQ = 1500000 + 10;
const int MAXN = 500 + 10;
const int INF  = 1e9;

const int BUFF_SIZE = 1e5;
int buffPos = BUFF_SIZE - 1;
char buff[BUFF_SIZE];

inline void readChar()
{
    if (++buffPos == BUFF_SIZE) fread(buff, BUFF_SIZE, 1, stdin), buffPos = 0;
}

inline void readInt(int &num)
{
    num = 0;
    for (; buff[buffPos] < '0' || buff[buffPos] > '9'; readChar());
    for (; buff[buffPos] >= '0' && buff[buffPos] <= '9'; readChar())
        num = 10 * num + buff[buffPos] - '0';
}

int n, q;
struct BIT
{
    int tree[MAXN];
    void reset()
    {
        std::fill(tree, tree + n + 1, 0);
    }

    inline void update(int idx, int val)
    {
        for (int pos = idx ; pos <= n ; pos += pos & (-pos))
        {
            tree[pos] += val;
        }
    }

    inline int query(int idx)
    {
        int ans = 0;
        for (int pos = idx ; pos > 0 ; pos -= pos & (-pos))
        {
            ans += tree[pos];
        }

        return ans;
    }
};

BIT tree;
struct Query
{
    int a, b, r, idx;
};

struct Query2
{
    int r, idx;
};

int sum;
int x[MAXN];
int ans[MAXQ];
int val[MAXS];
int ptr[MAXS];
std::bitset <MAXS> prefix[MAXN], bs;
std::vector <Query> queries[MAXN];
std::vector <Query2> toAns[MAXS];
std::vector <int> possible;

void answerQueries(int l)
{
    int max = 0;
    for (const int &i : possible)
    {
        max = std::max(max, ptr[i] - 1);
    }

    prefix[l][0] = 1;
    for (int i = l ; i <= max ; ++i)
    {
        prefix[i] |= prefix[i] << x[l];
    }

    if (queries[l].empty())
    {
        return;
    }

    for (int i = 1 ; i <= sum ; ++i)
    {
        val[i] = n + 1;
        toAns[i].clear();
    }

    for (const int &k : possible)
    {
        while (ptr[k] > l && prefix[ptr[k] - 1][k])
        {
            ptr[k]--;
        }

        val[k] = ptr[k];
    }

    if (n > 300) return;
    for (const Query &curr : queries[l])
    {
        toAns[curr.b].push_back({curr.r, curr.idx});
        toAns[curr.a].push_back({-curr.r, curr.idx});
    }

    tree.reset();
    for (int i = 1 ; i <= sum ; ++i)
    {
        int lastVal = -1, lastRes;
        tree.update(val[i], 1);
        for (const Query2 &curr : toAns[i])
        {
            if (abs(curr.r) != lastVal)
            {
                lastRes = tree.query(abs(curr.r));
                lastVal = abs(curr.r);
            }

            if (curr.r < 0) ans[curr.idx] -= lastRes;
            else ans[curr.idx] += lastRes;
        }
    }
}

void solve()
{
    bs[0] = 1;
    for (int i = 1 ; i <= n ; ++i)
    {
        bs |= bs << x[i];
    }

    for (int i = 1 ; i <= sum ; ++i)
    {
        if (bs[i])
        {
            possible.push_back(i);
        }
    }

    std::fill(ptr + 1, ptr + sum + 1, n + 1);
    for (int i = n ; i >= 1 ; --i)
    {
        answerQueries(i);
    }
}

void output()
{
    for (int i = 1 ; i <= q ; ++i)
    {
        std::cout << ans[i] << '\n';
    }
}

void input()
{
    readInt(n);
    readInt(q);
    for (int i = 1 ; i <= n ; ++i)
    {
        readInt(x[i]);
        sum += x[i];
    }

    int l, r, a, b;
    for (int j = 1 ; j <= q ; ++j)
    {
        readInt(l);
        readInt(r);
        readInt(a);
        readInt(b);
        if (a > sum) continue;
        a--; b = std::min(b, sum);
        queries[l].push_back({a, b, r, j});
    }
}

void fastIO()
{
    std::ios_base :: sync_with_stdio(0);
    std::cout.tie(nullptr);
    std::cin.tie(nullptr);
}

int main()
{
    fastIO();
    input();
    solve();
    output();

    return 0;
}
