#include <algorithm>
#include <iostream>
#include <numeric>
#include <cassert>
#include <vector>
#include <bitset>

#pragma GCC optimize ("O3,unroll_loops")

typedef long long llong;
const int MAXS = 250000 + 10;
const int MAXQ = 1500000 + 10;
const int MAXN = 500 + 10;
const int INF  = 1e9;

const int BUFF_SIZE = 1e5;
int buffPos = BUFF_SIZE - 1;
char buff[BUFF_SIZE];

inline void readChar()
{
    if (++buffPos == BUFF_SIZE) fread(buff, BUFF_SIZE, 1, stdin), buffPos = 0;
}

inline void readInt(int &num)
{
    num = 0;
    for (; buff[buffPos] < '0' || buff[buffPos] > '9'; readChar());
    for (; buff[buffPos] >= '0' && buff[buffPos] <= '9'; readChar())
        num = 10 * num + buff[buffPos] - '0';
}

int n, q;
int tree[MAXN];
void reset()
{
    std::fill(tree, tree + n + 1, 0);
}

inline void update(int idx)
{
    for (int pos = idx ; pos <= n ; pos += pos & (-pos))
    {
        tree[pos]++;
    }
}

inline int query(int idx)
{
    int ans = 0;
    for (int pos = idx ; pos > 0 ; pos -= pos & (-pos))
    {
        ans += tree[pos];
    }

    return ans;
}

struct Query
{
    int to, r, idx;
    inline friend bool operator < (const Query &a, const Query &b)
    {
        return a.to < b.to;
    }
};

int sum;
int x[MAXN];
int ans[MAXQ];
int val[MAXS];
int ptr[MAXS];
std::bitset <MAXS> prefix[MAXN], bs;
std::vector <Query> queries[MAXN];
std::vector <int> possible;

void solve()
{
    bs[0] = 1;
    for (int i = 1 ; i <= n ; ++i)
    {
        bs |= bs << x[i];
    }

    for (int i = 1 ; i <= sum ; ++i)
    {
        if (bs[i])
        {
            possible.push_back(i);
        }
    }

    int sum2 = 0;
    std::fill(ptr + 1, ptr + sum + 1, n + 1);
    std::fill(val + 1, val + sum + 1, n + 1);
    for (int l = n ; l >= 1 ; --l)
    {
        sum2 += x[l];
        int max = 0;
        for (const int &i : possible)
        {
            if (i > sum2) break;
            max = std::max(max, ptr[i] - 1);
        }

        prefix[l][0] = 1;
        for (int i = l ; i <= max ; ++i)
        {
            prefix[i] |= prefix[i] << x[l];
        }

        if (queries[l].empty())
        {
            continue;
        }

        for (const int &k : possible)
        {
            if (k > sum2) break;
            while (ptr[k] > l && prefix[ptr[k] - 1][k])
            {
                ptr[k]--;
            }

            val[k] = ptr[k];
        }

        int reached = 0; reset();
        std::sort(queries[l].begin(), queries[l].end());
        for (const Query &curr : queries[l])
        {
            while (reached < curr.to)
            {
                update(ptr[++reached]);
            }

            if (curr.r < 0) ans[curr.idx] -= query(-curr.r);
            else ans[curr.idx] += query(curr.r);
        }
    }
}

void output()
{
    for (int i = 1 ; i <= q ; ++i)
    {
        std::cout << ans[i] << '\n';
    }
}

void input()
{
    readInt(n);
    readInt(q);
    for (int i = 1 ; i <= n ; ++i)
    {
        readInt(x[i]);
        sum += x[i];
    }

    int l, r, a, b;
    for (int j = 1 ; j <= q ; ++j)
    {
        readInt(l);
        readInt(r);
        readInt(a);
        readInt(b);
        if (a > sum) continue;
        a--; b = std::min(b, sum);
        queries[l].push_back({a, -r, j});
        queries[l].push_back({b, r, j});
    }
}

void fastIO()
{
    std::ios_base :: sync_with_stdio(0);
    std::cout.tie(nullptr);
    std::cin.tie(nullptr);
}

int main()
{
    fastIO();
    input();
    solve();
    output();

    return 0;
}
